classdef SCDsignal
  
  properties
    Value                  {mustBeNumericOrLogical,mustBeNonempty} = 0;
    QualityTag       (1,1) QualityTag                              = QualityTag.INVALID;
    ProductionState  (1,1) ProductionState                         = ProductionState.STOPPED;
  end
  
  methods
    function S = SCDsignal(varargin)      
      % constructor
      if numel(varargin)>=1 && ~isempty(varargin{1}), S.Value           = varargin{1}; end
      if numel(varargin)>=2 && ~isempty(varargin{2}), S.QualityTag      = varargin{2}; end
      if numel(varargin)>=3 && ~isempty(varargin{3}), S.ProductionState = varargin{3}; end
      
      siz = S.datasize;
      assert(~(siz(1)==1 && siz(2)>1),'SCDsignal:RowVector',...
        'Row vectors are not supported due to matlab bug. Size=[%d %d]',siz(1),siz(2));
      % to reproduce the bug:
      % a=timeseries([1 2],1); Simulink.SimulationData.TimeseriesUtil.getSampleDimensions(a), a
    end
    
    function type = datatype(obj)
      type = class(obj.Value);
      assert(ismember(type,{'single', 'boolean', 'int8','int16','int32'}),...
        sprintf('DataType %s is not supported in SCDalgo', type))
    end
    
    function str = struct(obj)
      % method to convert to structure
      for ii=fieldnames(obj)'
        myf = ii{:};
        str.(myf) = obj.(myf);
      end
    end
    
    function dimension = dimension(obj)
      % dimension as required by buses etc.
      siz = obj.datasize;
      if siz(2) == 1, dimension = siz(1);
      else,          dimension = siz;
      end
    end
      
    function siz = datasize(obj,dim)
      % size of data
      if nargin==1, siz = size(obj.Value);
      else, siz = size(obj.Value,dim);  
      end
    end
    
    function valid = isvalid(obj)
      % true if GOOD and RUNNING
      isRunning = ([obj.ProductionState] == 'RUNNING');
      isGood    = ([obj.QualityTag]     == 'GOOD');
      valid = isRunning & isGood;
    end
    
    function [BusesObj] = createBus(obj)
      BusesObj = repmat(Simulink.Bus,size(obj));
      if numel(obj)>1 % for arrays of SCDsignal objects
        for ii = 1:numel(obj)
          BusesObj(ii) = createBus(obj(ii)); % recursive call
        end
      else
        % creates a bus object corresponding to this signal class
        tmp = Simulink.Bus.createObject(struct(obj));
        myBusObj = evalin('base',tmp.busName);
        evalin('base',sprintf('clear %s',tmp.busName));
        myBusObj.Description = obj.BusName;
        % 2D to 1D where possible
        for iEl = 1:numel(myBusObj.Elements)
          dims =  myBusObj.Elements(iEl).Dimensions;
          if dims(2) == 1
            myBusObj.Elements(iEl).Dimensions = dims(1);
          end
        end
        BusesObj = myBusObj;
      end
    end
    
    function Name = BusName(obj)
      % Returns the bus name for this signal
      type = obj.datatype;
      siz  = obj.datasize;
      assert(numel(siz)==2,'3 or larger dimensional data not supported')
      if all(siz == 1)
        sizestr = '_Scalar';
      elseif siz(2) == 1
        sizestr = sprintf('_%d_Vector',siz(1));
      else % matrix
        sizestr = sprintf('_%d_%d_Matrix',siz(1),siz(2));
      end
      Name = sprintf('SCDBus_%s%s',type,sizestr);
    end
    
  end
  
end